@@ -20,3 +20,8 @@
  */
 
 #include <drivers/atmodem/atutil.h>
+
+extern void cdma_voicecall_init(void);
+extern void cdma_voicecall_exit(void);
+extern void cdma_devinfo_init(void);
+extern void cdma_devinfo_exit(void);
