@@ -205,7 +205,7 @@ static struct pppcp_packet *pppcp_packet_new(struct pppcp_data *data,
 	guint16 packet_length = bufferlen + sizeof(*packet);
 
 	ppp_packet = ppp_packet_new(packet_length, data->driver->proto);
-	if (!ppp_packet)
+	if (ppp_packet == NULL)
 		return NULL;
 
 	/* advance past protocol to add CP header information */
@@ -997,12 +997,12 @@ void pppcp_set_local_options(struct pppcp_data *pppcp,
 }
 
 struct pppcp_data *pppcp_new(GAtPPP *ppp, const struct pppcp_proto *proto,
-				gboolean dormant)
+				gboolean dormant, guint max_failure)
 {
 	struct pppcp_data *data;
 
 	data = g_try_malloc0(sizeof(struct pppcp_data));
-	if (!data)
+	if (data == NULL)
 		return NULL;
 
 	if (dormant)
@@ -1016,7 +1016,11 @@ struct pppcp_data *pppcp_new(GAtPPP *ppp, const struct pppcp_proto *proto,
 	data->terminate_timer_data.max_counter = MAX_TERMINATE;
 	data->config_timer_data.data = data;
 	data->terminate_timer_data.data = data;
-	data->max_failure = MAX_FAILURE;
+
+	if (max_failure)
+		data->max_failure = max_failure;
+	else
+		data->max_failure = MAX_FAILURE;
 
 	data->ppp = ppp;
 	data->driver = proto;
