@@ -73,6 +73,7 @@ struct _GIsiModem {
 struct _GIsiPending {
 	enum GIsiMessageType type;
 	GIsiServiceMux *service;
+	gpointer owner;
 	guint timeout;
 	GIsiNotifyFunc notify;
 	GDestroyNotify destroy;
@@ -158,6 +159,34 @@ static void pending_dispatch(GIsiPending *pend, GIsiMessage *msg)
 	pend->notify(msg, pend->data);
 }
 
+static void pending_remove_and_dispatch(GIsiPending *op, GIsiMessage *msg)
+{
+	GIsiModem *modem;
+
+	op->service->pending = g_slist_remove(op->service->pending, op);
+
+	if (op->notify == NULL || msg == NULL)
+		goto destroy;
+
+	modem = op->service->modem;
+
+	ISIDBG(modem, "%s %s to %p [res=0x%02X, id=0x%02X, utid=0x%02X]",
+		g_isi_msg_strerror(msg), pend_type_to_str(op->type), op,
+		g_isi_msg_resource(msg), g_isi_msg_id(msg),
+		g_isi_msg_utid(msg));
+
+	op->notify(msg, op->data);
+
+destroy:
+	if (op->timeout > 0)
+		g_source_remove(op->timeout);
+
+	if (op->destroy != NULL)
+		op->destroy(op->data);
+
+	g_free(op);
+}
+
 static void service_dispatch(GIsiServiceMux *mux, GIsiMessage *msg,
 				gboolean is_indication)
 {
@@ -169,7 +198,6 @@ static void service_dispatch(GIsiServiceMux *mux, GIsiMessage *msg,
 	while (l != NULL) {
 		GSList *next = l->next;
 		GIsiPending *pend = l->data;
-		msg->private = pend;
 
 		/*
 		 * REQs, NTFs and INDs are dispatched on message ID.  While
@@ -194,22 +222,16 @@ static void service_dispatch(GIsiServiceMux *mux, GIsiMessage *msg,
 		} else if (pend->type == GISI_MESSAGE_TYPE_RESP &&
 				!is_indication && pend->utid == utid) {
 
-			pending_dispatch(pend, msg);
-			pend->notify = NULL;
-
-			g_isi_pending_remove(pend);
+			pending_remove_and_dispatch(pend, msg);
 			break;
 
 		} else if (pend->type == GISI_MESSAGE_TYPE_COMMON &&
 				msgid == COMMON_MESSAGE &&
 				pend->msgid == COMM_ISI_VERSION_GET_REQ) {
 
-			pending_dispatch(pend, msg);
-			pend->notify = NULL;
-
-			g_isi_pending_remove(pend);
-
+			pending_remove_and_dispatch(pend, msg);
 		}
+
 		l = next;
 	}
 }
@@ -634,16 +656,15 @@ static void vtrace(struct sockaddr_pn *dst,
 
 static gboolean resp_timeout(gpointer data)
 {
-	GIsiPending *resp = data;
+	GIsiPending *op = data;
 	GIsiMessage msg = {
 		.error = ETIMEDOUT,
-		.private = resp,
 	};
 
-	pending_dispatch(resp, &msg);
-	resp->notify = NULL;
+	op->timeout = 0;
+
+	pending_remove_and_dispatch(op, &msg);
 
-	g_isi_pending_remove(resp);
 	return FALSE;
 }
 
@@ -745,18 +766,11 @@ uint8_t g_isi_request_utid(GIsiPending *resp)
 	return resp != NULL ? resp->utid : 0;
 }
 
-GIsiPending *g_isi_pending_from_msg(const GIsiMessage *msg)
-{
-	return msg != NULL ? msg->private : NULL;
-}
-
 void g_isi_pending_remove(GIsiPending *op)
 {
 	if (op == NULL)
 		return;
 
-	op->service->pending = g_slist_remove(op->service->pending, op);
-
 	if (op->type == GISI_MESSAGE_TYPE_IND)
 		service_subs_decr(op->service);
 
@@ -766,15 +780,79 @@ void g_isi_pending_remove(GIsiPending *op)
 	if (op->type == GISI_MESSAGE_TYPE_RESP && op->notify != NULL) {
 		GIsiMessage msg = {
 			.error = ESHUTDOWN,
-			.private = op,
 		};
-		op->notify(&msg, op->data);
-		op->notify = NULL;
+
+		pending_remove_and_dispatch(op, &msg);
+		return;
+	}
+
+	op->service->pending = g_slist_remove(op->service->pending, op);
+
+	pending_destroy(op, NULL);
+}
+
+static void foreach_destroy(GIsiPending *op)
+{
+	if (op->type == GISI_MESSAGE_TYPE_IND)
+		service_subs_decr(op->service);
+
+	if (op->type == GISI_MESSAGE_TYPE_REQ)
+		service_regs_decr(op->service);
+
+	if (op->type == GISI_MESSAGE_TYPE_RESP && op->notify != NULL) {
+		GIsiMessage msg = {
+			.error = ESHUTDOWN,
+		};
+
+		pending_dispatch(op, &msg);
 	}
 
 	pending_destroy(op, NULL);
 }
 
+void g_isi_pending_set_owner(GIsiPending *op, gpointer owner)
+{
+	if (op == NULL)
+		return;
+
+	op->owner = owner;
+}
+
+void g_isi_remove_pending_by_owner(GIsiModem *modem, uint8_t resource,
+					gpointer owner)
+{
+	GIsiServiceMux *mux;
+	GSList *l;
+	GSList *next;
+	GIsiPending *op;
+	GSList *owned = NULL;
+
+	mux = service_get(modem, resource);
+	if (mux == NULL)
+		return;
+
+	for (l = mux->pending; l != NULL; l = next) {
+		next = l->next;
+		op = l->data;
+
+		if (op->owner != owner)
+			continue;
+
+		mux->pending = g_slist_remove_link(mux->pending, l);
+
+		l->next = owned;
+		owned = l;
+	}
+
+	for (l = owned; l != NULL; l = l->next) {
+		op = l->data;
+
+		foreach_destroy(op);
+	}
+
+	g_slist_free(owned);
+}
+
 GIsiPending *g_isi_ntf_subscribe(GIsiModem *modem, uint8_t resource,
 					uint8_t msgid, GIsiNotifyFunc notify,
 					void *data, GDestroyNotify destroy)
@@ -911,6 +989,29 @@ int g_isi_response_vsend(GIsiModem *modem, const GIsiMessage *req,
 	return g_isi_modem_vsendto(modem, req->addr, _iov, 1 + iovlen);
 }
 
+int g_isi_modem_send(GIsiModem *modem, uint8_t resource,
+			const void *__restrict buf, size_t len)
+{
+	struct sockaddr_pn dst = {
+		.spn_family = AF_PHONET,
+		.spn_resource = resource,
+	};
+
+	return g_isi_modem_sendto(modem, &dst, buf, len);
+}
+
+int g_isi_modem_vsend(GIsiModem *modem, uint8_t resource,
+				const struct iovec *__restrict iov,
+				size_t iovlen)
+{
+	struct sockaddr_pn dst = {
+		.spn_family = AF_PHONET,
+		.spn_resource = resource,
+	};
+
+	return g_isi_modem_vsendto(modem, &dst, iov, iovlen);
+}
+
 int g_isi_modem_sendto(GIsiModem *modem, struct sockaddr_pn *dst,
 			const void *__restrict buf, size_t len)
 {
@@ -1022,14 +1123,11 @@ static gboolean reachable_notify(gpointer data)
 	};
 	GIsiMessage msg = {
 		.version = &mux->version,
-		.private = pong,
 		.addr = &addr,
 	};
 
-	pending_dispatch(pong, &msg);
-	pong->notify = NULL;
+	pending_remove_and_dispatch(pong, &msg);
 
-	g_isi_pending_remove(pong);
 	return FALSE;
 }
 
