@@ -65,7 +65,7 @@ static gboolean g_isi_pep_callback(GIOChannel *channel, GIOCondition cond,
 	}
 	pep->gprs_fd = fd;
 
-	if (pep->ready)
+	if (pep->ready != NULL)
 		pep->ready(pep, pep->opaque);
 
 	return FALSE;
