@@ -163,7 +163,7 @@ static void cm_get_properties_reply(struct ofono_call_meter *cm)
 	const char *currency = cm->currency;
 
 	reply = dbus_message_new_method_return(cm->pending);
-	if (!reply)
+	if (reply == NULL)
 		return;
 
 	dbus_message_iter_init_append(reply, &iter);
@@ -181,7 +181,8 @@ static void cm_get_properties_reply(struct ofono_call_meter *cm)
 	ofono_dbus_dict_append(&dict, "AccumulatedCallMeterMaximum",
 				DBUS_TYPE_UINT32, &cm->acm_max);
 
-	ofono_dbus_dict_append(&dict, "PricePerUnit", DBUS_TYPE_DOUBLE, &cm->ppu);
+	ofono_dbus_dict_append(&dict, "PricePerUnit", DBUS_TYPE_DOUBLE,
+				&cm->ppu);
 
 	ofono_dbus_dict_append(&dict, "Currency", DBUS_TYPE_STRING, &currency);
 
@@ -190,8 +191,8 @@ static void cm_get_properties_reply(struct ofono_call_meter *cm)
 	__ofono_dbus_pending_reply(&cm->pending, reply);
 }
 
-static void query_call_meter_callback(const struct ofono_error *error, int value,
-					void *data)
+static void query_call_meter_callback(const struct ofono_error *error,
+					int value, void *data)
 {
 	struct ofono_call_meter *cm = data;
 
@@ -204,7 +205,7 @@ static void query_call_meter_callback(const struct ofono_error *error, int value
 
 static void query_call_meter(struct ofono_call_meter *cm)
 {
-	if (!cm->driver->call_meter_query) {
+	if (cm->driver->call_meter_query == NULL) {
 		if (cm->pending)
 			cm_get_properties_reply(cm);
 
@@ -227,7 +228,7 @@ static void query_acm_callback(const struct ofono_error *error, int value,
 
 static void query_acm(struct ofono_call_meter *cm)
 {
-	if (!cm->driver->acm_query) {
+	if (cm->driver->acm_query == NULL) {
 		query_call_meter(cm);
 		return;
 	}
@@ -250,7 +251,7 @@ static void query_acm_max_callback(const struct ofono_error *error, int value,
 
 static void query_acm_max(struct ofono_call_meter *cm)
 {
-	if (!cm->driver->acm_max_query) {
+	if (cm->driver->acm_max_query == NULL) {
 		cm->flags |= CALL_METER_FLAG_CACHED;
 
 		query_acm(cm);
@@ -276,7 +277,7 @@ static void query_puct_callback(const struct ofono_error *error,
 
 static void query_puct(struct ofono_call_meter *cm)
 {
-	if (!cm->driver->puct_query)
+	if (cm->driver->puct_query == NULL)
 		query_acm_max(cm);
 	else
 		cm->driver->puct_query(cm, query_puct_callback, cm);
@@ -292,7 +293,8 @@ static DBusMessage *cm_get_properties(DBusConnection *conn, DBusMessage *msg,
 
 	cm->pending = dbus_message_ref(msg);
 
-	/* We don't need to query ppu, currency & acm_max every time
+	/*
+	 * We don't need to query ppu, currency & acm_max every time
 	 * Not sure if we have to query acm & call_meter every time
 	 * so lets play on the safe side and query them.  They should be
 	 * fast to query anyway
@@ -305,13 +307,13 @@ static DBusMessage *cm_get_properties(DBusConnection *conn, DBusMessage *msg,
 	return NULL;
 }
 
-static void set_acm_max_query_callback(const struct ofono_error *error, int value,
-					void *data)
+static void set_acm_max_query_callback(const struct ofono_error *error,
+					int value, void *data)
 {
 	struct ofono_call_meter *cm = data;
 	DBusMessage *reply;
 
-	if (!cm->pending)
+	if (cm->pending == NULL)
 		return;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
@@ -352,7 +354,7 @@ static DBusMessage *prop_set_acm_max(DBusMessage *msg,
 {
 	dbus_uint32_t value;
 
-	if (!cm->driver->acm_max_set)
+	if (cm->driver->acm_max_set == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	dbus_message_iter_get_basic(dbus_value, &value);
@@ -371,7 +373,7 @@ static void set_puct_query_callback(const struct ofono_error *error,
 	struct ofono_call_meter *cm = data;
 	DBusMessage *reply;
 
-	if (!cm->pending)
+	if (cm->pending == NULL)
 		return;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
@@ -406,7 +408,8 @@ static void set_puct_callback(const struct ofono_error *error, void *data)
 	cm->driver->puct_query(cm, set_puct_query_callback, cm);
 }
 
-/* This function is for the really bizarre case of someone trying to call
+/*
+ * This function is for the really bizarre case of someone trying to call
  * SetProperty before GetProperties.  But we must handle it...
  */
 static void set_puct_initial_query_callback(const struct ofono_error *error,
@@ -419,7 +422,7 @@ static void set_puct_initial_query_callback(const struct ofono_error *error,
 	const char *name;
 	const char *pin2;
 
-	if (!cm->pending)
+	if (cm->pending == NULL)
 		return;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
@@ -454,7 +457,7 @@ static DBusMessage *prop_set_ppu(DBusMessage *msg, struct ofono_call_meter *cm,
 {
 	double ppu;
 
-	if (!cm->driver->puct_set || !cm->driver->puct_query)
+	if (cm->driver->puct_set == NULL || cm->driver->puct_query == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	dbus_message_iter_get_basic(var, &ppu);
@@ -478,7 +481,7 @@ static DBusMessage *prop_set_cur(DBusMessage *msg, struct ofono_call_meter *cm,
 {
 	const char *value;
 
-	if (!cm->driver->puct_set || !cm->driver->puct_query)
+	if (cm->driver->puct_set == NULL || cm->driver->puct_query == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	dbus_message_iter_get_basic(var, &value);
@@ -568,7 +571,7 @@ static void reset_acm_query_callback(const struct ofono_error *error, int value,
 	struct ofono_call_meter *cm = data;
 	DBusMessage *reply;
 
-	if (!cm->pending)
+	if (cm->pending == NULL)
 		return;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
@@ -608,7 +611,7 @@ static DBusMessage *cm_acm_reset(DBusConnection *conn, DBusMessage *msg,
 	struct ofono_call_meter *cm = data;
 	const char *pin2;
 
-	if (!cm->driver->acm_reset)
+	if (cm->driver->acm_reset == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	if (cm->pending)
@@ -665,7 +668,7 @@ int ofono_call_meter_driver_register(const struct ofono_call_meter_driver *d)
 	if (d->probe == NULL)
 		return -EINVAL;
 
-	g_drivers = g_slist_prepend(g_drivers, (void *)d);
+	g_drivers = g_slist_prepend(g_drivers, (void *) d);
 
 	return 0;
 }
@@ -674,7 +677,7 @@ void ofono_call_meter_driver_unregister(const struct ofono_call_meter_driver *d)
 {
 	DBG("driver: %p, name: %s", d, d->name);
 
-	g_drivers = g_slist_remove(g_drivers, (void *)d);
+	g_drivers = g_slist_remove(g_drivers, (void *) d);
 }
 
 static void call_meter_unregister(struct ofono_atom *atom)
