@@ -121,7 +121,7 @@ unsigned int __ofono_modem_callid_next(struct ofono_modem *modem)
 	unsigned int i;
 
 	for (i = 1; i < sizeof(modem->call_ids) * 8; i++) {
-		if (modem->call_ids & (0x1 << i))
+		if (modem->call_ids & (1 << i))
 			continue;
 
 		return i;
@@ -132,12 +132,12 @@ unsigned int __ofono_modem_callid_next(struct ofono_modem *modem)
 
 void __ofono_modem_callid_hold(struct ofono_modem *modem, int id)
 {
-	modem->call_ids |= (0x1 << id);
+	modem->call_ids |= (1 << id);
 }
 
 void __ofono_modem_callid_release(struct ofono_modem *modem, int id)
 {
-	modem->call_ids &= ~(0x1 << id);
+	modem->call_ids &= ~(1 << id);
 }
 
 void ofono_modem_set_data(struct ofono_modem *modem, void *data)
@@ -580,15 +580,15 @@ static DBusMessage *set_property_online(struct ofono_modem *modem,
 	if (modem->pending != NULL)
 		return __ofono_error_busy(msg);
 
+	if (modem->online == online)
+		return dbus_message_new_method_return(msg);
+
 	if (driver->set_online == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	if (modem->modem_state < MODEM_STATE_OFFLINE)
 		return __ofono_error_not_available(msg);
 
-	if (modem->online == online)
-		return dbus_message_new_method_return(msg);
-
 	modem->pending = dbus_message_ref(msg);
 
 	driver->set_online(modem, online,
@@ -736,6 +736,8 @@ static void lockdown_remove(struct ofono_modem *modem)
 	}
 
 	g_free(modem->lock_owner);
+	modem->lock_owner = NULL;
+
 	modem->lockdown = FALSE;
 }
 
@@ -794,6 +796,7 @@ static DBusMessage *set_property_lockdown(struct ofono_modem *modem,
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
 	ofono_bool_t lockdown;
+	dbus_bool_t powered;
 	const char *caller;
 	int err;
 
@@ -805,54 +808,55 @@ static DBusMessage *set_property_lockdown(struct ofono_modem *modem,
 	if (modem->pending != NULL)
 		return __ofono_error_busy(msg);
 
+	caller = dbus_message_get_sender(msg);
+
+	if (modem->lockdown && g_strcmp0(caller, modem->lock_owner))
+		return __ofono_error_access_denied(msg);
+
 	if (modem->lockdown == lockdown)
 		return dbus_message_new_method_return(msg);
 
-	caller = dbus_message_get_sender(msg);
+	if (lockdown == FALSE) {
+		lockdown_remove(modem);
+		goto done;
+	}
 
-	if (lockdown) {
-		dbus_bool_t powered;
+	modem->lock_owner = g_strdup(caller);
 
-		modem->lock_owner = g_strdup(caller);
-
-		modem->lock_watch = g_dbus_add_disconnect_watch(conn,
+	modem->lock_watch = g_dbus_add_disconnect_watch(conn,
 				modem->lock_owner, lockdown_disconnect,
 				modem, NULL);
 
-		if (modem->lock_watch == 0) {
-			g_free(modem->lock_owner);
+	if (modem->lock_watch == 0) {
+		g_free(modem->lock_owner);
+		modem->lock_owner = NULL;
+
+		return __ofono_error_failed(msg);
+	}
+
+	modem->lockdown = lockdown;
+
+	if (modem->powered == FALSE)
+		goto done;
+
+	err = set_powered(modem, FALSE);
+	if (err < 0) {
+		if (err != -EINPROGRESS) {
+			lockdown_remove(modem);
 			return __ofono_error_failed(msg);
 		}
 
-		modem->lockdown = lockdown;
-
-		if (!modem->powered)
-			goto done;
-
-		err = set_powered(modem, FALSE);
-		if (err < 0) {
-			if (err != -EINPROGRESS) {
-				lockdown_remove(modem);
-				return __ofono_error_failed(msg);
-			}
-
-			modem->pending = dbus_message_ref(msg);
-			modem->timeout = g_timeout_add_seconds(20,
+		modem->pending = dbus_message_ref(msg);
+		modem->timeout = g_timeout_add_seconds(20,
 						set_powered_timeout, modem);
-			return NULL;
-		}
+		return NULL;
+	}
 
-		powered = FALSE;
-		ofono_dbus_signal_property_changed(conn, modem->path,
+	powered = FALSE;
+	ofono_dbus_signal_property_changed(conn, modem->path,
 					OFONO_MODEM_INTERFACE,
 					"Powered", DBUS_TYPE_BOOLEAN,
 					&powered);
-	} else {
-		if (g_strcmp0(caller, modem->lock_owner))
-			return __ofono_error_access_denied(msg);
-
-		lockdown_remove(modem);
-	}
 
 done:
 	g_dbus_send_reply(conn, msg, DBUS_TYPE_INVALID);
@@ -1075,8 +1079,10 @@ static const struct {
 	{ OFONO_MESSAGE_MANAGER_INTERFACE,		"sms"	},
 	{ OFONO_SIM_MANAGER_INTERFACE,			"sim"	},
 	{ OFONO_STK_INTERFACE,				"stk"	},
+	{ OFONO_SUPPLEMENTARY_SERVICES_INTERFACE,	"ussd"	},
 	{ OFONO_CONNECTION_MANAGER_INTERFACE,		"gprs"	},
 	{ OFONO_TEXT_TELEPHONY_INTERFACE,		"tty"	},
+	{ OFONO_LOCATION_REPORTING_INTERFACE,		"gps"	},
 	{ },
 };
 
@@ -1560,12 +1566,12 @@ static void sim_watch(struct ofono_atom *atom,
 							modem, NULL);
 }
 
-void __ofono_modemwatch_init()
+void __ofono_modemwatch_init(void)
 {
 	g_modemwatches = __ofono_watchlist_new(g_free);
 }
 
-void __ofono_modemwatch_cleanup()
+void __ofono_modemwatch_cleanup(void)
 {
 	__ofono_watchlist_free(g_modemwatches);
 }
@@ -1850,7 +1856,7 @@ void ofono_modem_driver_unregister(const struct ofono_modem_driver *d)
 	}
 }
 
-void __ofono_modem_shutdown()
+void __ofono_modem_shutdown(void)
 {
 	struct ofono_modem *modem;
 	GSList *l;
