@@ -62,6 +62,8 @@ struct ofono_sim {
 	enum ofono_sim_password_type pin_type;
 	gboolean locked_pins[OFONO_SIM_PASSWORD_SIM_PUK]; /* Number of PINs */
 
+	int pin_retries[OFONO_SIM_PASSWORD_INVALID];
+
 	enum ofono_sim_phase phase;
 	unsigned char mnc_length;
 	enum ofono_sim_cphs_phase cphs_phase;
@@ -76,6 +78,8 @@ struct ofono_sim {
 	gboolean barred_dialing;
 
 	char *imsi;
+	char mcc[OFONO_MAX_MCC_LENGTH + 1];
+	char mnc[OFONO_MAX_MNC_LENGTH + 1];
 
 	GSList *own_numbers;
 	GSList *new_numbers;
@@ -92,6 +96,7 @@ struct ofono_sim {
 	struct ofono_watchlist *state_watches;
 
 	struct sim_fs *simfs;
+	struct ofono_sim_context *context;
 
 	unsigned char *iidf_image;
 
@@ -248,6 +253,31 @@ static char **get_locked_pins(struct ofono_sim *sim)
 	return ret;
 }
 
+static void **get_pin_retries(struct ofono_sim *sim)
+{
+	int i, nelem;
+	void **ret;
+
+	for (i = 1, nelem = 0; i < OFONO_SIM_PASSWORD_INVALID; i++) {
+		if (sim->pin_retries[i] == -1)
+			continue;
+
+		nelem += 1;
+	}
+
+	ret = g_new0(void *, nelem * 2 + 1);
+
+	for (i = 1, nelem = 0; i < OFONO_SIM_PASSWORD_INVALID; i++) {
+		if (sim->pin_retries[i] == -1)
+			continue;
+
+		ret[nelem++] = (void *) sim_passwd_name(i);
+		ret[nelem++] = &sim->pin_retries[i];
+	}
+
+	return ret;
+}
+
 static char **get_service_numbers(GSList *service_numbers)
 {
 	int nelem;
@@ -287,12 +317,13 @@ static DBusMessage *sim_get_properties(DBusConnection *conn,
 	char **service_numbers;
 	char **locked_pins;
 	const char *pin_name;
+	void **pin_retries;
 	dbus_bool_t present = sim->state != OFONO_SIM_STATE_NOT_PRESENT;
 	dbus_bool_t fdn;
 	dbus_bool_t bdn;
 
 	reply = dbus_message_new_method_return(msg);
-	if (!reply)
+	if (reply == NULL)
 		return NULL;
 
 	dbus_message_iter_init_append(reply, &iter);
@@ -320,21 +351,13 @@ static DBusMessage *sim_get_properties(DBusConnection *conn,
 	bdn = sim->barred_dialing;
 	ofono_dbus_dict_append(&dict, "BarredDialing", DBUS_TYPE_BOOLEAN, &bdn);
 
-	if (sim->mnc_length && sim->imsi) {
-		char mcc[OFONO_MAX_MCC_LENGTH + 1];
-		char mnc[OFONO_MAX_MNC_LENGTH + 1];
+	if (sim->mcc[0] != '\0' && sim->mnc[0] != '\0') {
 		const char *str;
-
-		strncpy(mcc, sim->imsi, OFONO_MAX_MCC_LENGTH);
-		mcc[OFONO_MAX_MCC_LENGTH] = '\0';
-		strncpy(mnc, sim->imsi + OFONO_MAX_MCC_LENGTH, sim->mnc_length);
-		mnc[sim->mnc_length] = '\0';
-
-		str = mcc;
+		str = sim->mcc;
 		ofono_dbus_dict_append(&dict, "MobileCountryCode",
 					DBUS_TYPE_STRING, &str);
 
-		str = mnc;
+		str = sim->mnc;
 		ofono_dbus_dict_append(&dict, "MobileNetworkCode",
 					DBUS_TYPE_STRING, &str);
 	}
@@ -369,12 +392,52 @@ static DBusMessage *sim_get_properties(DBusConnection *conn,
 				DBUS_TYPE_STRING,
 				(void *) &pin_name);
 
+	pin_retries = get_pin_retries(sim);
+	ofono_dbus_dict_append_dict(&dict, "Retries", DBUS_TYPE_BYTE,
+								&pin_retries);
+	g_free(pin_retries);
+
 done:
 	dbus_message_iter_close_container(&iter, &dict);
 
 	return reply;
 }
 
+static void sim_pin_retries_query_cb(const struct ofono_error *error,
+					int retries[OFONO_SIM_PASSWORD_INVALID],
+					void *data)
+{
+	struct ofono_sim *sim = data;
+	DBusConnection *conn = ofono_dbus_get_connection();
+	const char *path = __ofono_atom_get_path(sim->atom);
+	void **pin_retries;
+
+	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
+		ofono_error("Querying remaining pin retries failed");
+
+		return;
+	}
+
+	if (!memcmp(retries, sim->pin_retries, sizeof(sim->pin_retries)))
+		return;
+
+	memcpy(sim->pin_retries, retries, sizeof(sim->pin_retries));
+
+	pin_retries = get_pin_retries(sim);
+	ofono_dbus_signal_dict_property_changed(conn, path,
+					OFONO_SIM_MANAGER_INTERFACE, "Retries",
+					DBUS_TYPE_BYTE,	&pin_retries);
+	g_free(pin_retries);
+}
+
+static void sim_pin_retries_check(struct ofono_sim *sim)
+{
+	if (sim->driver->query_pin_retries == NULL)
+		return;
+
+	sim->driver->query_pin_retries(sim, sim_pin_retries_query_cb, sim);
+}
+
 static void msisdn_set_done(struct msisdn_set_request *req)
 {
 	DBusMessage *reply;
@@ -429,9 +492,11 @@ static gboolean set_own_numbers(struct ofono_sim *sim,
 			new_numbers = new_numbers->next;
 		} else {
 			memset(efmsisdn, 0xff, sim->efmsisdn_length);
+			/* Set number length */
+			efmsisdn[sim->efmsisdn_length - 14] = 1;
 		}
 
-		if (ofono_sim_write(req->sim, SIM_EFMSISDN_FILEID,
+		if (ofono_sim_write(req->sim->context, SIM_EFMSISDN_FILEID,
 				msisdn_set_cb, OFONO_SIM_FILE_STRUCTURE_FIXED,
 				record, efmsisdn,
 				sim->efmsisdn_length, req) == 0)
@@ -547,6 +612,8 @@ static void sim_locked_cb(struct ofono_sim *sim, gboolean locked)
 						"LockedPins", DBUS_TYPE_STRING,
 						&locked_pins);
 	g_strfreev(locked_pins);
+
+	sim_pin_retries_check(sim);
 }
 
 static void sim_unlock_cb(const struct ofono_error *error, void *data)
@@ -555,7 +622,10 @@ static void sim_unlock_cb(const struct ofono_error *error, void *data)
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
 		DBusMessage *reply = __ofono_error_failed(sim->pending);
+
 		__ofono_dbus_pending_reply(&sim->pending, reply);
+		sim_pin_retries_check(sim);
+
 		return;
 	}
 
@@ -568,7 +638,10 @@ static void sim_lock_cb(const struct ofono_error *error, void *data)
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
 		DBusMessage *reply = __ofono_error_failed(sim->pending);
+
 		__ofono_dbus_pending_reply(&sim->pending, reply);
+		sim_pin_retries_check(sim);
+
 		return;
 	}
 
@@ -582,7 +655,7 @@ static DBusMessage *sim_lock_or_unlock(struct ofono_sim *sim, int lock,
 	const char *typestr;
 	const char *pin;
 
-	if (!sim->driver->lock)
+	if (sim->driver->lock == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	if (sim->pending)
@@ -637,11 +710,16 @@ static void sim_change_pin_cb(const struct ofono_error *error, void *data)
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
 		__ofono_dbus_pending_reply(&sim->pending,
 				__ofono_error_failed(sim->pending));
+
+		sim_pin_retries_check(sim);
+
 		return;
 	}
 
 	__ofono_dbus_pending_reply(&sim->pending,
 				dbus_message_new_method_return(sim->pending));
+
+	sim_pin_retries_check(sim);
 }
 
 static DBusMessage *sim_change_pin(DBusConnection *conn, DBusMessage *msg,
@@ -653,7 +731,7 @@ static DBusMessage *sim_change_pin(DBusConnection *conn, DBusMessage *msg,
 	const char *old;
 	const char *new;
 
-	if (!sim->driver->change_passwd)
+	if (sim->driver->change_passwd == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	if (sim->pending)
@@ -709,7 +787,7 @@ static DBusMessage *sim_enter_pin(DBusConnection *conn, DBusMessage *msg,
 	enum ofono_sim_password_type type;
 	const char *pin;
 
-	if (!sim->driver->send_passwd)
+	if (sim->driver->send_passwd == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	if (sim->pending)
@@ -846,7 +924,7 @@ static void sim_iidf_read_cb(int ok, int length, int record,
 	sim->iidf_image = g_memdup(data, length);
 
 	/* read the clut data */
-	ofono_sim_read_bytes(sim, iidf_id, offset, clut_len,
+	ofono_sim_read_bytes(sim->context, iidf_id, offset, clut_len,
 					sim_iidf_read_clut_cb, sim);
 }
 
@@ -878,7 +956,7 @@ static void sim_get_image(struct ofono_sim *sim, unsigned char id,
 	iidf_len = efimg[7] << 8 | efimg[8];
 
 	/* read the image data */
-	ofono_sim_read_bytes(sim, iidf_id, iidf_offset, iidf_len,
+	ofono_sim_read_bytes(sim->context, iidf_id, iidf_offset, iidf_len,
 				sim_iidf_read_cb, sim);
 }
 
@@ -918,7 +996,7 @@ static DBusMessage *sim_reset_pin(DBusConnection *conn, DBusMessage *msg,
 	const char *puk;
 	const char *pin;
 
-	if (!sim->driver->reset_passwd)
+	if (sim->driver->reset_passwd == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	if (sim->pending)
@@ -976,7 +1054,7 @@ static gboolean numbers_list_equal(GSList *a, GSList *b)
 	struct ofono_phone_number *num_a, *num_b;
 
 	while (a || b) {
-		if (!a || !b)
+		if (a == NULL || b == NULL)
 			return FALSE;
 
 		num_a = a->data;
@@ -1097,7 +1175,7 @@ static void sim_sdn_read_cb(int ok, int length, int record,
 	if (sim->service_numbers &&
 			g_slist_find_custom(sim->service_numbers,
 				alpha, service_number_compare)) {
-		ofono_error("Duplicate EFsdn entries for `%s'\n",
+		ofono_error("Duplicate EFsdn entries for `%s'",
 				alpha);
 		g_free(alpha);
 
@@ -1135,8 +1213,9 @@ check:
 
 static void sim_own_numbers_update(struct ofono_sim *sim)
 {
-	ofono_sim_read(sim, SIM_EFMSISDN_FILEID, OFONO_SIM_FILE_STRUCTURE_FIXED,
-			sim_msisdn_read_cb, sim);
+	ofono_sim_read(sim->context, SIM_EFMSISDN_FILEID,
+			OFONO_SIM_FILE_STRUCTURE_FIXED, sim_msisdn_read_cb,
+			sim);
 }
 
 static void sim_efimg_read_cb(int ok, int length, int record,
@@ -1189,10 +1268,10 @@ static void sim_ready(enum ofono_sim_state new_state, void *user)
 
 	sim_own_numbers_update(sim);
 
-	ofono_sim_read(sim, SIM_EFSDN_FILEID, OFONO_SIM_FILE_STRUCTURE_FIXED,
-			sim_sdn_read_cb, sim);
-	ofono_sim_read(sim, SIM_EFIMG_FILEID, OFONO_SIM_FILE_STRUCTURE_FIXED,
-			sim_efimg_read_cb, sim);
+	ofono_sim_read(sim->context, SIM_EFSDN_FILEID,
+			OFONO_SIM_FILE_STRUCTURE_FIXED, sim_sdn_read_cb, sim);
+	ofono_sim_read(sim->context, SIM_EFIMG_FILEID,
+			OFONO_SIM_FILE_STRUCTURE_FIXED, sim_efimg_read_cb, sim);
 }
 
 static void sim_imsi_cb(const struct ofono_error *error, const char *imsi,
@@ -1215,22 +1294,21 @@ static void sim_imsi_cb(const struct ofono_error *error, const char *imsi,
 						DBUS_TYPE_STRING, &sim->imsi);
 
 	if (sim->mnc_length) {
-		char mcc[OFONO_MAX_MCC_LENGTH + 1];
-		char mnc[OFONO_MAX_MNC_LENGTH + 1];
 		const char *str;
 
-		strncpy(mcc, sim->imsi, OFONO_MAX_MCC_LENGTH);
-		mcc[OFONO_MAX_MCC_LENGTH] = '\0';
-		strncpy(mnc, sim->imsi + OFONO_MAX_MCC_LENGTH, sim->mnc_length);
-		mnc[sim->mnc_length] = '\0';
+		strncpy(sim->mcc, sim->imsi, OFONO_MAX_MCC_LENGTH);
+		sim->mcc[OFONO_MAX_MCC_LENGTH] = '\0';
+		strncpy(sim->mnc, sim->imsi + OFONO_MAX_MCC_LENGTH,
+			sim->mnc_length);
+		sim->mnc[sim->mnc_length] = '\0';
 
-		str = mcc;
+		str = sim->mcc;
 		ofono_dbus_signal_property_changed(conn, path,
 						OFONO_SIM_MANAGER_INTERFACE,
 						"MobileCountryCode",
 						DBUS_TYPE_STRING, &str);
 
-		str = mnc;
+		str = sim->mnc;
 		ofono_dbus_signal_property_changed(conn, path,
 						OFONO_SIM_MANAGER_INTERFACE,
 						"MobileNetworkCode",
@@ -1242,7 +1320,7 @@ static void sim_imsi_cb(const struct ofono_error *error, const char *imsi,
 
 static void sim_retrieve_imsi(struct ofono_sim *sim)
 {
-	if (!sim->driver->read_imsi) {
+	if (sim->driver->read_imsi == NULL) {
 		ofono_error("IMSI retrieval not implemented,"
 				" only emergency calls will be available");
 		return;
@@ -1309,7 +1387,7 @@ static gboolean check_bdn_status(struct ofono_sim *sim)
 	 */
 	if (sim_sst_is_active(sim->efsst, sim->efsst_length,
 			SIM_SST_SERVICE_BDN)) {
-		sim_fs_read_info(sim->simfs, SIM_EFBDN_FILEID,
+		sim_fs_read_info(sim->context, SIM_EFBDN_FILEID,
 				OFONO_SIM_FILE_STRUCTURE_FIXED,
 				sim_efbdn_info_read_cb, sim);
 		return TRUE;
@@ -1363,7 +1441,7 @@ static void sim_efsst_read_cb(int ok, int length, int record,
 	 */
 	if (sim_sst_is_active(sim->efsst, sim->efsst_length,
 				SIM_SST_SERVICE_FDN)) {
-		sim_fs_read_info(sim->simfs, SIM_EFADN_FILEID,
+		sim_fs_read_info(sim->context, SIM_EFADN_FILEID,
 					OFONO_SIM_FILE_STRUCTURE_FIXED,
 					sim_efadn_info_read_cb, sim);
 		return;
@@ -1441,12 +1519,19 @@ static void sim_efust_read_cb(int ok, int length, int record,
 
 	/*
 	 * Check whether the SIM provides EFest file
-	 * According to 31.102, section 4.2.24 and 4.2.44 the EFest file
-	 * must be present if EFfdn or EFbdn are present
+	 * According to 3GPP TS 31.102 section 4.2.47, EFest file
+	 * shall be present if FDN or BDN or EST is available
+	 * Lets be paranoid and check for the special cases as well
+	 * where EST is not available(FDN or BDN available), but EFest
+	 * is present
 	 */
 	if (sim_ust_is_available(sim->efust, sim->efust_length,
-				SIM_UST_SERVICE_ENABLED_SERVICE_TABLE)) {
-		ofono_sim_read(sim, SIM_EFEST_FILEID,
+				SIM_UST_SERVICE_ENABLED_SERVICE_TABLE) ||
+			sim_ust_is_available(sim->efust, sim->efust_length,
+				SIM_UST_SERVICE_FDN) ||
+			sim_ust_is_available(sim->efust, sim->efust_length,
+				SIM_UST_SERVICE_BDN)) {
+		ofono_sim_read(sim->context, SIM_EFEST_FILEID,
 				OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
 				sim_efest_read_cb, sim);
 
@@ -1507,7 +1592,7 @@ static void sim_efphase_read_cb(int ok, int length, int record,
 	if (!ok || length != 1) {
 		sim->phase = OFONO_SIM_PHASE_3G;
 
-		ofono_sim_read(sim, SIM_EFUST_FILEID,
+		ofono_sim_read(sim->context, SIM_EFUST_FILEID,
 				OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
 				sim_efust_read_cb, sim);
 
@@ -1529,18 +1614,18 @@ static void sim_efphase_read_cb(int ok, int length, int record,
 		return;
 	}
 
-	ofono_sim_read(sim, SIM_EFSST_FILEID,
+	ofono_sim_read(sim->context, SIM_EFSST_FILEID,
 			OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
 			sim_efsst_read_cb, sim);
 }
 
 static void sim_initialize_after_pin(struct ofono_sim *sim)
 {
-	ofono_sim_read(sim, SIM_EFPHASE_FILEID,
+	ofono_sim_read(sim->context, SIM_EFPHASE_FILEID,
 			OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
 			sim_efphase_read_cb, sim);
 
-	ofono_sim_read(sim, SIM_EFAD_FILEID,
+	ofono_sim_read(sim->context, SIM_EFAD_FILEID,
 			OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
 			sim_ad_read_cb, sim);
 
@@ -1548,7 +1633,7 @@ static void sim_initialize_after_pin(struct ofono_sim *sim)
 	 * Read CPHS-support bits, this is still part of the SIM
 	 * initialisation but no order is specified for it.
 	 */
-	ofono_sim_read(sim, SIM_EF_CPHS_INFORMATION_FILEID,
+	ofono_sim_read(sim->context, SIM_EF_CPHS_INFORMATION_FILEID,
 			OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
 			sim_cphs_information_read_cb, sim);
 }
@@ -1585,6 +1670,8 @@ static void sim_pin_query_cb(const struct ofono_error *error,
 						&pin_name);
 	}
 
+	sim_pin_retries_check(sim);
+
 checkdone:
 	if (pin_type == OFONO_SIM_PASSWORD_NONE)
 		sim_initialize_after_pin(sim);
@@ -1592,7 +1679,7 @@ checkdone:
 
 static void sim_pin_check(struct ofono_sim *sim)
 {
-	if (!sim->driver->query_passwd_state) {
+	if (sim->driver->query_passwd_state == NULL) {
 		sim_initialize_after_pin(sim);
 		return;
 	}
@@ -1838,7 +1925,7 @@ static void sim_initialize(struct ofono_sim *sim)
 	 */
 
 	/* Grab the EFiccid which is always available */
-	ofono_sim_read(sim, SIM_EF_ICCID_FILEID,
+	ofono_sim_read(sim->context, SIM_EF_ICCID_FILEID,
 			OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
 			sim_iccid_read_cb, sim);
 
@@ -1852,47 +1939,51 @@ static void sim_initialize(struct ofono_sim *sim)
 	 * However we don't depend on the user interface and so
 	 * need to read both files now.
 	 */
-	ofono_sim_read(sim, SIM_EFLI_FILEID,
+	ofono_sim_read(sim->context, SIM_EFLI_FILEID,
 			OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
 			sim_efli_read_cb, sim);
-	ofono_sim_read(sim, SIM_EFPL_FILEID,
+	ofono_sim_read(sim->context, SIM_EFPL_FILEID,
 			OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
 			sim_efpl_read_cb, sim);
 }
 
-int ofono_sim_read_bytes(struct ofono_sim *sim, int id,
+struct ofono_sim_context *ofono_sim_context_create(struct ofono_sim *sim)
+{
+	if (sim == NULL || sim->simfs == NULL)
+		return NULL;
+
+	return sim_fs_context_new(sim->simfs);
+}
+
+void ofono_sim_context_free(struct ofono_sim_context *context)
+{
+	return sim_fs_context_free(context);
+}
+
+int ofono_sim_read_bytes(struct ofono_sim_context *context, int id,
 			unsigned short offset, unsigned short num_bytes,
 			ofono_sim_file_read_cb_t cb, void *data)
 {
-	if (sim == NULL)
-		return -1;
-
 	if (num_bytes == 0)
 		return -1;
 
-	return sim_fs_read(sim->simfs, id, OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
+	return sim_fs_read(context, id, OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
 				offset, num_bytes, cb, data);
 }
 
-int ofono_sim_read(struct ofono_sim *sim, int id,
+int ofono_sim_read(struct ofono_sim_context *context, int id,
 			enum ofono_sim_file_structure expected_type,
 			ofono_sim_file_read_cb_t cb, void *data)
 {
-	if (sim == NULL)
-		return -1;
-
-	return sim_fs_read(sim->simfs, id, expected_type, 0, 0, cb, data);
+	return sim_fs_read(context, id, expected_type, 0, 0, cb, data);
 }
 
-int ofono_sim_write(struct ofono_sim *sim, int id,
+int ofono_sim_write(struct ofono_sim_context *context, int id,
 			ofono_sim_file_write_cb_t cb,
 			enum ofono_sim_file_structure structure, int record,
 			const unsigned char *data, int length, void *userdata)
 {
-	if (sim == NULL)
-		return -1;
-
-	return sim_fs_write(sim->simfs, id, cb, structure, record, data, length,
+	return sim_fs_write(context, id, cb, structure, record, data, length,
 				userdata);
 }
 
@@ -1904,10 +1995,26 @@ const char *ofono_sim_get_imsi(struct ofono_sim *sim)
 	return sim->imsi;
 }
 
+const char *ofono_sim_get_mcc(struct ofono_sim *sim)
+{
+	if (sim == NULL)
+		return NULL;
+
+	return sim->mcc;
+}
+
+const char *ofono_sim_get_mnc(struct ofono_sim *sim)
+{
+	if (sim == NULL)
+		return NULL;
+
+	return sim->mnc;
+}
+
 enum ofono_sim_phase ofono_sim_get_phase(struct ofono_sim *sim)
 {
 	if (sim == NULL)
-		return 0;
+		return OFONO_SIM_PHASE_UNKNOWN;
 
 	return sim->phase;
 }
@@ -1967,6 +2074,9 @@ static void sim_free_state(struct ofono_sim *sim)
 		sim->imsi = NULL;
 	}
 
+	sim->mcc[0] = '\0';
+	sim->mnc[0] = '\0';
+
 	if (sim->own_numbers) {
 		g_slist_foreach(sim->own_numbers, (GFunc)g_free, NULL);
 		g_slist_free(sim->own_numbers);
@@ -2155,16 +2265,19 @@ static void sim_remove(struct ofono_atom *atom)
 	if (sim == NULL)
 		return;
 
-	if (sim->driver && sim->driver->remove)
+	if (sim->driver != NULL && sim->driver->remove != NULL)
 		sim->driver->remove(sim);
 
 	sim_free_state(sim);
 
-	if (sim->simfs) {
-		sim_fs_free(sim->simfs);
-		sim->simfs = NULL;
+	if (sim->context) {
+		ofono_sim_context_free(sim->context);
+		sim->context = NULL;
 	}
 
+	sim_fs_free(sim->simfs);
+	sim->simfs = NULL;
+
 	g_free(sim);
 }
 
@@ -2175,6 +2288,7 @@ struct ofono_sim *ofono_sim_create(struct ofono_modem *modem,
 {
 	struct ofono_sim *sim;
 	GSList *l;
+	int i;
 
 	if (driver == NULL)
 		return NULL;
@@ -2188,6 +2302,9 @@ struct ofono_sim *ofono_sim_create(struct ofono_modem *modem,
 	sim->atom = __ofono_modem_add_atom(modem, OFONO_ATOM_TYPE_SIM,
 						sim_remove, sim);
 
+	for (i = 0; i < OFONO_SIM_PASSWORD_INVALID; i++)
+		sim->pin_retries[i] = -1;
+
 	for (l = g_drivers; l; l = l->next) {
 		const struct ofono_sim_driver *drv = l->data;
 
@@ -2223,6 +2340,7 @@ void ofono_sim_register(struct ofono_sim *sim)
 	ofono_modem_add_interface(modem, OFONO_SIM_MANAGER_INTERFACE);
 	sim->state_watches = __ofono_watchlist_new(g_free);
 	sim->simfs = sim_fs_new(sim, sim->driver);
+	sim->context = ofono_sim_context_create(sim);
 
 	__ofono_atom_register(sim->atom, sim_unregister);
 
