@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -24,24 +24,32 @@ enum gsm_dialect {
 	GSM_DIALECT_TURKISH,
 	GSM_DIALECT_SPANISH,
 	GSM_DIALECT_PORTUGUESE,
-	GSM_DIALECT_INVALID
 };
 
 char *convert_gsm_to_utf8(const unsigned char *text, long len, long *items_read,
 				long *items_written, unsigned char terminator);
 
-char *convert_gsm_to_utf8_with_lang(const unsigned char *text, long len, long *items_read,
-				long *items_written, unsigned char terminator,
-				enum gsm_dialect locking_shift_lang,
-				enum gsm_dialect single_shift_lang);
+char *convert_gsm_to_utf8_with_lang(const unsigned char *text, long len,
+					long *items_read, long *items_written,
+					unsigned char terminator,
+					enum gsm_dialect locking_shift_lang,
+					enum gsm_dialect single_shift_lang);
 
 unsigned char *convert_utf8_to_gsm(const char *text, long len, long *items_read,
 				long *items_written, unsigned char terminator);
 
-unsigned char *convert_utf8_to_gsm_with_lang(const char *text, long len, long *items_read,
-				long *items_written, unsigned char terminator,
-				enum gsm_dialect locking_shift_lang,
-				enum gsm_dialect single_shift_lang);
+unsigned char *convert_utf8_to_gsm_with_lang(const char *text, long len,
+					long *items_read, long *items_written,
+					unsigned char terminator,
+					enum gsm_dialect locking_shift_lang,
+					enum gsm_dialect single_shift_lang);
+
+unsigned char *convert_utf8_to_gsm_best_lang(const char *utf8, long len,
+					long *items_read, long *items_written,
+					unsigned char terminator,
+					enum gsm_dialect hint,
+					enum gsm_dialect *used_locking,
+					enum gsm_dialect *used_single);
 
 unsigned char *decode_hex_own_buf(const char *in, long len, long *items_written,
 					unsigned char terminator,
@@ -77,3 +85,17 @@ unsigned char *pack_7bit(const unsigned char *in, long len, int byte_offset,
 				long *items_written, unsigned char terminator);
 
 char *sim_string_to_utf8(const unsigned char *buffer, int length);
+
+unsigned char *utf8_to_sim_string(const char *utf,
+					int max_length, int *out_length);
+
+unsigned char *convert_ucs2_to_gsm_with_lang(const unsigned char *text,
+						long len, long *items_read,
+						long *items_written,
+						unsigned char terminator,
+						enum gsm_dialect locking_lang,
+						enum gsm_dialect single_lang);
+
+unsigned char *convert_ucs2_to_gsm(const unsigned char *text, long len,
+					long *items_read, long *items_written,
+					unsigned char terminator);
